@@ -14,6 +14,14 @@ def download_sha1_unzip(url, checksum, save_to_directory, unzip=True):
     Does not download again if the file is there.
     Does not unzip again if the file is there.
     """
+    # requests does connection retrying, but people might not have it installed.
+    use_requests = True
+
+    try:
+        import requests
+    except ImportError:
+        use_requests = False
+
     try:
         import urllib.request as urllib
     except ImportError:
@@ -34,17 +42,27 @@ def download_sha1_unzip(url, checksum, save_to_directory, unzip=True):
                 print("Skipping download url:%s: save_to:%s:" % (url, save_to))
     else:
         print("Downloading...", url, checksum)
-        headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, '
-                                 'like Gecko) Chrome/35.0.1916.47 Safari/537.36'}
-        request = urllib.Request(url, headers=headers)
-        response = urllib.urlopen(request).read()
-        cont_checksum = hashlib.sha1(response).hexdigest()
+
+        if use_requests:
+            response = requests.get(url)
+            cont_checksum = hashlib.sha1(response.content).hexdigest()
+        else:
+
+            headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, '
+                                     'like Gecko) Chrome/35.0.1916.47 Safari/537.36'}
+            request = urllib.Request(url, headers=headers)
+            response = urllib.urlopen(request).read()
+            cont_checksum = hashlib.sha1(response).hexdigest()
+
         if checksum != cont_checksum:
             raise ValueError(
                 'url:%s should have checksum:%s: Has:%s: ' % (url, checksum, cont_checksum)
             )
         with open(save_to, 'wb') as f:
-            f.write(response)
+            if use_requests:
+                f.write(response.content)
+            else:
+                f.write(response)
 
     if unzip and filename.endswith('.zip'):
         print("Unzipping :%s:" % save_to)
@@ -64,8 +82,8 @@ def get_urls(x86=True, x64=True, sdl2=True):
     if sdl2:
         url_sha1.extend([
             [
-            'https://www.libsdl.org/release/SDL2-devel-2.0.14-VC.zip',
-            '48d5dcd4a445410301f5575219cffb6de654edb8',
+            'https://www.libsdl.org/release/SDL2-devel-2.0.16-VC.zip',
+            '13d952c333f3c2ebe9b7bc0075b4ad2f784e7584',
             ],
             [
             'https://www.libsdl.org/projects/SDL_image/release/SDL2_image-devel-2.0.5-VC.zip',
@@ -80,7 +98,8 @@ def get_urls(x86=True, x64=True, sdl2=True):
             '9097148f4529cf19f805ccd007618dec280f0ecc',
             ],
             [
-            'https://www.ijg.org/files/jpegsr9d.zip',
+            # 'https://www.ijg.org/files/jpegsr9d.zip',
+            'https://www.pygame.org/ftp/jpegsr9d.zip',
             'ed10aa2b5a0fcfe74f8a6f7611aeb346b06a1f99',
             ],
         ])
@@ -99,10 +118,9 @@ def get_urls(x86=True, x64=True, sdl2=True):
 def download_prebuilts(temp_dir, x86=True, x64=True, sdl2=True):
     """ For downloading prebuilt dependencies.
     """
-    from distutils.dir_util import mkpath
     if not os.path.exists(temp_dir):
         print("Making dir :%s:" % temp_dir)
-        mkpath(temp_dir)
+        os.makedirs(temp_dir)
     for url, checksum in get_urls(x86=x86, x64=x64, sdl2=sdl2):
         download_sha1_unzip(url, checksum, temp_dir, 1)
 
@@ -244,12 +262,12 @@ def place_downloaded_prebuilts(temp_dir, move_to_dir, x86=True, x64=True, sdl2=T
         copy(
             os.path.join(
                 temp_dir,
-                'SDL2-devel-2.0.14-VC/SDL2-2.0.14'
+                'SDL2-devel-2.0.16-VC/SDL2-2.0.16'
             ),
             os.path.join(
                 move_to_dir,
                 prebuilt_dir,
-                'SDL2-2.0.14'
+                'SDL2-2.0.16'
             )
         )
 
