@@ -90,6 +90,8 @@ static PyObject *
 _color_lerp(pgColorObject *, PyObject *, PyObject *);
 static PyObject *
 _premul_alpha(pgColorObject *, PyObject *);
+static PyObject *
+_color_update(pgColorObject *, PyObject *, PyObject *);
 
 /* Getters/setters */
 static PyObject *
@@ -203,6 +205,8 @@ static PyMethodDef _color_methods[] = {
      DOC_COLORLERP},
     {"premul_alpha", (PyCFunction)_premul_alpha, METH_NOARGS,
      DOC_COLORPREMULALPHA},
+    {"update", (PyCFunction)_color_update, METH_VARARGS,
+     DOC_COLORUPDATE},
     {NULL, NULL, 0, NULL}};
 
 /**
@@ -322,7 +326,8 @@ static PyBufferProcs _color_as_buffer = {
 #define DEFERRED_ADDRESS(ADDR) 0
 
 static PyTypeObject pgColor_Type = {
-    TYPE_HEAD(NULL, 0) "pygame.Color", /* tp_name */
+    PyVarObject_HEAD_INIT(NULL,0)
+    "pygame.Color",                    /* tp_name */
     sizeof(pgColorObject),             /* tp_basicsize */
     0,                                 /* tp_itemsize */
     (destructor)_color_dealloc,        /* tp_dealloc */
@@ -920,6 +925,52 @@ _premul_alpha(pgColorObject *color, PyObject *args)
     return (PyObject *)_color_new_internal(Py_TYPE(color), new_rgba);
 }
 
+static PyObject *
+_color_update(pgColorObject *self, PyObject *args, PyObject *kwargs)
+{
+    Uint8 *rgba = self->data;
+    PyObject *r_or_obj;
+    PyObject *g = NULL;
+    PyObject *b = NULL;
+    PyObject *a = NULL;
+
+    if (!PyArg_ParseTuple(args, "O|OOO", &r_or_obj, &g, &b, &a)) {
+        return NULL;
+    }
+
+    if (!g) {
+        if (_parse_color_from_single_object(r_or_obj, rgba)) {
+            return NULL;
+        }
+    }
+    else {
+        Uint32 color = 0;
+
+        /* Color(R,G,B[,A]) */
+        if (!_get_color(r_or_obj, &color) || color > 255) {
+            return RAISE(PyExc_ValueError, "invalid color argument");
+        }
+        rgba[0] = (Uint8)color;
+        if (!_get_color(g, &color) || color > 255) {
+            return RAISE(PyExc_ValueError, "invalid color argument");
+        }
+        rgba[1] = (Uint8)color;
+        if (!b || !_get_color(b, &color) || color > 255) {
+            return RAISE(PyExc_ValueError, "invalid color argument");
+        }
+        rgba[2] = (Uint8)color;
+
+        if (a) {
+            if (!_get_color(a, &color) || color > 255) {
+                return RAISE(PyExc_ValueError, "invalid color argument");
+            }
+            self->len = 4;
+            rgba[3] = (Uint8)color;
+        }
+    }
+    Py_RETURN_NONE;
+}
+
 /**
  * color.r
  */
@@ -1071,6 +1122,13 @@ _color_get_hsva(pgColorObject *color, void *closure)
     /* Calculate S */
     hsv[1] = 100. * (maxv - minv) / maxv;
 
+    /* Clamp S, needed on some but not all FPUs */
+    if (hsv[1] < 0) {
+        hsv[1] = 0.f;
+    } else if (hsv[1] > 100) {
+        hsv[1] = 100.f;
+    }
+
     /* Calculate H */
     if (maxv == frgb[0]) {
         hsv[0] = fmod((60 * ((frgb[1] - frgb[2]) / diff)), 360.f);
