@@ -394,28 +394,22 @@ MODINIT_DEFINE(constants)
     DEC_CONST(TEXTINPUT);
     DEC_CONST(TEXTEDITING);
 
-    DEC_CONST(WINDOWEVENT);
-    DEC_CONST(WINDOWEVENT_SHOWN);
-    DEC_CONST(WINDOWEVENT_HIDDEN);
-    DEC_CONST(WINDOWEVENT_EXPOSED);
-    DEC_CONST(WINDOWEVENT_MOVED);
-    DEC_CONST(WINDOWEVENT_RESIZED);
-    DEC_CONST(WINDOWEVENT_SIZE_CHANGED);
-    DEC_CONST(WINDOWEVENT_MINIMIZED);
-    DEC_CONST(WINDOWEVENT_MAXIMIZED);
-    DEC_CONST(WINDOWEVENT_RESTORED);
-    DEC_CONST(WINDOWEVENT_ENTER);
-    DEC_CONST(WINDOWEVENT_LEAVE);
-    DEC_CONST(WINDOWEVENT_FOCUS_GAINED);
-    DEC_CONST(WINDOWEVENT_FOCUS_LOST);
-    DEC_CONST(WINDOWEVENT_CLOSE);
-    #if SDL_VERSION_ATLEAST(2, 0, 5)
-        DEC_CONST(WINDOWEVENT_TAKE_FOCUS);
-        DEC_CONST(WINDOWEVENT_HIT_TEST);
-    #else
-        DEC_CONSTS(WINDOWEVENT_TAKE_FOCUS, -1);
-        DEC_CONSTS(WINDOWEVENT_HIT_TEST, -1);
-    #endif /* SDL_VERSION_ATLEAST(2, 0, 5) */
+    DEC_CONSTS(WINDOWSHOWN, PGE_WINDOWSHOWN);
+    DEC_CONSTS(WINDOWHIDDEN, PGE_WINDOWHIDDEN);
+    DEC_CONSTS(WINDOWEXPOSED, PGE_WINDOWEXPOSED);
+    DEC_CONSTS(WINDOWMOVED, PGE_WINDOWMOVED);
+    DEC_CONSTS(WINDOWRESIZED, PGE_WINDOWRESIZED);
+    DEC_CONSTS(WINDOWSIZECHANGED, PGE_WINDOWSIZECHANGED);
+    DEC_CONSTS(WINDOWMINIMIZED, PGE_WINDOWMINIMIZED);
+    DEC_CONSTS(WINDOWMAXIMIZED, PGE_WINDOWMAXIMIZED);
+    DEC_CONSTS(WINDOWRESTORED, PGE_WINDOWRESTORED);
+    DEC_CONSTS(WINDOWENTER, PGE_WINDOWENTER);
+    DEC_CONSTS(WINDOWLEAVE, PGE_WINDOWLEAVE);
+    DEC_CONSTS(WINDOWFOCUSGAINED, PGE_WINDOWFOCUSGAINED);
+    DEC_CONSTS(WINDOWFOCUSLOST, PGE_WINDOWFOCUSLOST);
+    DEC_CONSTS(WINDOWCLOSE, PGE_WINDOWCLOSE);
+    DEC_CONSTS(WINDOWTAKEFOCUS, PGE_WINDOWTAKEFOCUS);
+    DEC_CONSTS(WINDOWHITTEST, PGE_WINDOWHITTEST);
 
     DEC_CONST(CONTROLLERAXISMOTION);
     DEC_CONST(CONTROLLERBUTTONDOWN);
@@ -423,6 +417,15 @@ MODINIT_DEFINE(constants)
     DEC_CONST(CONTROLLERDEVICEADDED);
     DEC_CONST(CONTROLLERDEVICEREMOVED);
     DEC_CONST(CONTROLLERDEVICEREMAPPED);
+#if SDL_VERSION_ATLEAST(2, 0, 14)
+    DEC_CONST(CONTROLLERTOUCHPADDOWN);
+    DEC_CONST(CONTROLLERTOUCHPADMOTION);
+    DEC_CONST(CONTROLLERTOUCHPADUP);
+#else
+    DEC_CONSTS(CONTROLLERTOUCHPADDOWN, -1);
+    DEC_CONSTS(CONTROLLERTOUCHPADMOTION, -1);
+    DEC_CONSTS(CONTROLLERTOUCHPADUP, -1);
+#endif
 
     DEC_CONST(JOYDEVICEADDED);
     DEC_CONST(JOYDEVICEREMOVED);
@@ -484,23 +487,22 @@ MODINIT_DEFINE(constants)
     DEC_CONSTS(TEXTINPUT, -1);
     DEC_CONSTS(TEXTEDITING, -1);
 
-    DEC_CONSTS(WINDOWEVENT, -1);
-    DEC_CONSTS(WINDOWEVENT_SHOWN, -1);
-    DEC_CONSTS(WINDOWEVENT_HIDDEN, -1);
-    DEC_CONSTS(WINDOWEVENT_EXPOSED, -1);
-    DEC_CONSTS(WINDOWEVENT_MOVED, -1);
-    DEC_CONSTS(WINDOWEVENT_RESIZED, -1);
-    DEC_CONSTS(WINDOWEVENT_SIZE_CHANGED, -1);
-    DEC_CONSTS(WINDOWEVENT_MINIMIZED, -1);
-    DEC_CONSTS(WINDOWEVENT_MAXIMIZED, -1);
-    DEC_CONSTS(WINDOWEVENT_RESTORED, -1);
-    DEC_CONSTS(WINDOWEVENT_ENTER, -1);
-    DEC_CONSTS(WINDOWEVENT_LEAVE, -1);
-    DEC_CONSTS(WINDOWEVENT_FOCUS_GAINED, -1);
-    DEC_CONSTS(WINDOWEVENT_FOCUS_LOST, -1);
-    DEC_CONSTS(WINDOWEVENT_CLOSE, -1);
-    DEC_CONSTS(WINDOWEVENT_TAKE_FOCUS, -1);
-    DEC_CONSTS(WINDOWEVENT_HIT_TEST, -1);
+    DEC_CONSTS(WINDOWSHOWN, -1);
+    DEC_CONSTS(WINDOWHIDDEN, -1);
+    DEC_CONSTS(WINDOWEXPOSED, -1);
+    DEC_CONSTS(WINDOWMOVED, -1);
+    DEC_CONSTS(WINDOWRESIZED, -1);
+    DEC_CONSTS(WINDOWSIZECHANGED, -1);
+    DEC_CONSTS(WINDOWMINIMIZED, -1);
+    DEC_CONSTS(WINDOWMAXIMIZED, -1);
+    DEC_CONSTS(WINDOWRESTORED, -1);
+    DEC_CONSTS(WINDOWENTER, -1);
+    DEC_CONSTS(WINDOWLEAVE, -1);
+    DEC_CONSTS(WINDOWFOCUSGAINED, -1);
+    DEC_CONSTS(WINDOWFOCUSLOST, -1);
+    DEC_CONSTS(WINDOWCLOSE, -1);
+    DEC_CONSTS(WINDOWTAKEFOCUS, -1);
+    DEC_CONSTS(WINDOWHITTEST, -1);
 
     DEC_CONSTS(CONTROLLERAXISMOTION, -1);
     DEC_CONSTS(CONTROLLERBUTTONDOWN, -1);
@@ -508,6 +510,9 @@ MODINIT_DEFINE(constants)
     DEC_CONSTS(CONTROLLERDEVICEADDED, -1);
     DEC_CONSTS(CONTROLLERDEVICEREMOVED, -1);
     DEC_CONSTS(CONTROLLERDEVICEREMAPPED, -1);
+    DEC_CONSTS(CONTROLLERTOUCHPADDOWN, -1);
+    DEC_CONSTS(CONTROLLERTOUCHPADMOTION, 1);
+    DEC_CONSTS(CONTROLLERTOUCHPADUP, -1);
 
     DEC_CONSTS(JOYDEVICEADDED, -1);
     DEC_CONSTS(JOYDEVICEREMOVED, -1);
@@ -719,6 +724,7 @@ MODINIT_DEFINE(constants)
     DEC_CONSTS(K_PERCENT, -1);
     DEC_CONSTS(K_PRINTSCREEN, -1);
     DEC_CONSTS(K_SCROLLLOCK, -1);
+    DEC_CONSTS(K_AC_BACK, -1);
 
     // scan code key codes (also not supported in SDL1)
     DEC_CONSTS(KSCAN_0, -1);
@@ -880,6 +886,7 @@ MODINIT_DEFINE(constants)
 #else  /* IS_SDLv2 */
     /* Keyboard key codes: Pygame K_ constants. Scan codes: KSCAN_ constants.
      */
+    DEC_CONSTK_AND_SC(AC_BACK);
     DEC_CONSTK_AND_SC(UNKNOWN);
     DEC_CONSTK_AND_SC(BACKSPACE);
     DEC_CONSTK_AND_SC(TAB);
