@@ -44,6 +44,13 @@
 #define FONT_HAVE_RWOPS 0
 #endif
 
+#ifndef SDL_TTF_VERSION_ATLEAST
+#define SDL_TTF_COMPILEDVERSION \
+    SDL_VERSIONNUM(SDL_TTF_MAJOR_VERSION, SDL_TTF_MINOR_VERSION, SDL_TTF_PATCHLEVEL)
+#define SDL_TTF_VERSION_ATLEAST(X, Y, Z) \
+    (SDL_TTF_COMPILEDVERSION >= SDL_VERSIONNUM(X, Y, Z))
+#endif
+
 #if PY3
 #define RAISE_TEXT_TYPE_ERROR() \
     RAISE(PyExc_TypeError, "text must be a unicode or bytes");
@@ -65,6 +72,7 @@ PyFont_New(TTF_Font *);
 #define PyFont_Check(x) ((x)->ob_type == &PyFont_Type)
 
 static int font_initialized = 0;
+static unsigned int current_ttf_generation = 0;
 static const char font_defaultname[] = "freesansbold.ttf";
 static const char pkgdatamodule_name[] = "pygame.pkgdata";
 static const char resourcefunc_name[] = "getResource";
@@ -200,6 +208,7 @@ fontmodule_quit(PyObject *self)
     if (font_initialized) {
         TTF_Quit();
         font_initialized = 0;
+        current_ttf_generation++;
     }
     Py_RETURN_NONE;
 }
@@ -511,12 +520,14 @@ font_render(PyObject *self, PyObject *args)
             return RAISE(PyExc_ValueError,
                          "A null character was found in the text");
         }
+#if !SDL_TTF_VERSION_ATLEAST(2, 0, 15)
         if (utf_8_needs_UCS_4(astring)) {
             Py_DECREF(bytes);
             return RAISE(PyExc_UnicodeError,
                          "A Unicode character above '\\uFFFF' was found;"
-                         " not supported");
+                         " not supported with SDL_ttf version below 2.0.15");
         }
+#endif
         if (aa) {
             if (bg_rgba_obj == NULL) {
                 surf = TTF_RenderUTF8_Blended(font, astring, foreg);
@@ -730,9 +741,16 @@ static void
 font_dealloc(PyFontObject *self)
 {
     TTF_Font *font = PyFont_AsFont(self);
-
-    if (font && font_initialized)
+    if (font && font_initialized) {
+        if (self->ttf_init_generation != current_ttf_generation) {
+            // Since TTF_Font is a private structure
+            // it's impossible to access face field in a common way.
+            int** face_pp = font;
+            *face_pp = NULL;
+        }
         TTF_CloseFont(font);
+        self->font = NULL;
+    }
 
     if (self->weakreflist)
         PyObject_ClearWeakRefs((PyObject *)self);
@@ -898,6 +916,7 @@ success:
     Py_XDECREF(oencoded);
     Py_DECREF(obj);
     self->font = font;
+    self->ttf_init_generation = current_ttf_generation;
     return 0;
 
 error:
@@ -1045,6 +1064,14 @@ MODINIT_DEFINE(font)
         MODINIT_ERROR;
     }
 
+#if SDL_TTF_VERSION_ATLEAST(2, 0, 15)
+    /* So people can check for UCS4 support. */
+    if (PyModule_AddIntConstant(module, "UCS4", 1)) {
+        DECREF_MOD(module);
+        MODINIT_ERROR;
+    }
+#endif
+
     /* export the c api */
     c_api[0] = &PyFont_Type;
     c_api[1] = PyFont_New;
