@@ -34,8 +34,19 @@ static PyObject *pgJoystick_New(int);
 static int _joy_map_insert(pgJoystickObject *jstick);
 #define pgJoystick_Check(x) ((x)->ob_type == &pgJoystick_Type)
 
-static void
-joy_autoquit(void)
+static PyObject *
+init(PyObject *self)
+{
+    if (!SDL_WasInit(SDL_INIT_JOYSTICK)) {
+        if (SDL_InitSubSystem(SDL_INIT_JOYSTICK))
+            return RAISE(pgExc_SDLError, SDL_GetError());
+        SDL_JoystickEventState(SDL_ENABLE);
+    }
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+quit(PyObject *self)
 {
     /* Walk joystick objects to deallocate the stick objects. */
     pgJoystickObject *cur = joylist_head;
@@ -51,41 +62,6 @@ joy_autoquit(void)
         SDL_JoystickEventState(SDL_ENABLE);
         SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
     }
-
-}
-
-static PyObject *
-joy_autoinit(PyObject *self)
-{
-    if (!SDL_WasInit(SDL_INIT_JOYSTICK)) {
-        if (SDL_InitSubSystem(SDL_INIT_JOYSTICK)) {
-            return PyInt_FromLong(0);
-        }
-        SDL_JoystickEventState(SDL_ENABLE);
-        pg_RegisterQuit(joy_autoquit);
-    }
-    return PyInt_FromLong(1);
-}
-
-static PyObject *
-quit(PyObject *self)
-{
-    joy_autoquit();
-    Py_RETURN_NONE;
-}
-
-static PyObject *
-init(PyObject *self)
-{
-    PyObject *result;
-    int istrue;
-
-    result = joy_autoinit(self);
-    istrue = PyObject_IsTrue(result);
-    Py_DECREF(result);
-    if (!istrue) {
-        return RAISE(pgExc_SDLError, SDL_GetError());
-    }
     Py_RETURN_NONE;
 }
 
@@ -285,8 +261,72 @@ joy_get_power_level(PyObject *self, PyObject *args)
     return Text_FromUTF8(leveltext);
 }
 
-#endif
+static PyObject *
+joy_rumble(pgJoystickObject *self, PyObject *args, PyObject *kwargs)
+{
+#if SDL_VERSION_ATLEAST(2, 0, 9)
 
+    SDL_Joystick *joy = self->joy;
+    float low;
+    float high;
+    uint32_t duration;
+    int res;
+
+    char *keywords[] = {
+        "low_frequency",
+        "high_frequency",
+        "duration",
+        NULL,
+    };
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "ffI", keywords, &low,
+                                     &high, &duration)) {
+        return NULL;
+    }
+
+    JOYSTICK_INIT_CHECK();
+    if (!joy) {
+        return RAISE(pgExc_SDLError, "Joystick not initialized");
+    }
+
+    if (low < 0) {
+        low = 0.f;
+    }
+    else if (low > 1.f) {
+        low = 1.f;
+    }
+
+    if (high < 0) {
+        high = 0.f;
+    }
+    else if (high > 1.f) {
+        high = 1.f;
+    }
+    low *= 0xFFFF;
+    high *= 0xFFFF;
+
+    res = SDL_JoystickRumble(joy, low, high, duration);
+    if (res == -1) {
+        Py_RETURN_FALSE;
+    }
+    Py_RETURN_TRUE;
+
+#else
+    Py_RETURN_FALSE;
+#endif
+}
+
+static PyObject *
+joy_stop_rumble(pgJoystickObject *self)
+{
+#if SDL_VERSION_ATLEAST(2, 0, 9)
+    SDL_Joystick *joy = self->joy;
+    SDL_JoystickRumble(joy, 0, 0, 1);
+#endif
+    Py_RETURN_NONE;
+}
+
+#endif /*if IS_SDLv2*/
 
 static PyObject *
 joy_get_name(PyObject *self, PyObject *args)
@@ -485,6 +525,8 @@ static PyMethodDef joy_methods[] = {
     {"get_instance_id", joy_get_instance_id, METH_NOARGS, DOC_JOYSTICKGETINSTANCEID},
     {"get_guid", joy_get_guid, METH_NOARGS, DOC_JOYSTICKGETGUID},
     {"get_power_level", joy_get_power_level, METH_NOARGS, DOC_JOYSTICKGETPOWERLEVEL},
+    {"rumble", (PyCFunction)joy_rumble, METH_VARARGS | METH_KEYWORDS, DOC_JOYSTICKRUMBLE},
+    {"stop_rumble", (PyCFunction)joy_stop_rumble, METH_NOARGS, DOC_JOYSTICKSTOPRUMBLE},
 #endif
     {"get_name", joy_get_name, METH_NOARGS, DOC_JOYSTICKGETNAME},
 
@@ -504,7 +546,8 @@ static PyMethodDef joy_methods[] = {
     {NULL, NULL, 0, NULL}};
 
 static PyTypeObject pgJoystick_Type = {
-    TYPE_HEAD(NULL, 0) "Joystick", /* name */
+    PyVarObject_HEAD_INIT(NULL,0)
+    "Joystick",                    /* name */
     sizeof(pgJoystickObject),      /* basic size */
     0,                             /* itemsize */
     joy_dealloc,                   /* dealloc */
@@ -593,8 +636,6 @@ pgJoystick_New(int id)
 }
 
 static PyMethodDef _joystick_methods[] = {
-    {"__PYGAMEinit__", (PyCFunction)joy_autoinit, METH_NOARGS,
-     "auto initialize function for joystick"},
     {"init", (PyCFunction)init, METH_NOARGS, DOC_PYGAMEJOYSTICKINIT},
     {"quit", (PyCFunction)quit, METH_NOARGS, DOC_PYGAMEJOYSTICKQUIT},
     {"get_init", (PyCFunction)get_init, METH_NOARGS,
