@@ -72,6 +72,9 @@ const PG_sample_format_t PG_SAMPLE_CHAR_SIGN = (char)0xff > 0 ? 0 : 0x10000u;
 #define PYGAME_MIXER_DEFAULT_ALLOWEDCHANGES -1
 #endif
 
+static int
+sound_init(PyObject *, PyObject *, PyObject *);
+
 static PyTypeObject pgSound_Type;
 static PyTypeObject pgChannel_Type;
 static PyObject *
@@ -93,9 +96,6 @@ static int request_chunksize = PYGAME_MIXER_DEFAULT_CHUNKSIZE;
 static int request_allowedchanges = PYGAME_MIXER_DEFAULT_ALLOWEDCHANGES;
 static char *request_devicename = NULL;
 
-static int
-sound_init(PyObject *self, PyObject *arg, PyObject *kwarg);
-
 struct ChannelData {
     PyObject *sound;
     PyObject *queue;
@@ -261,18 +261,45 @@ _format_view_to_audio(Py_buffer *view)
     return format;
 }
 
+static void
+_pg_push_mixer_event(int type, int code)
+{
+    pgEventObject *e;
+    PyObject *dict, *dictcode;
+    SDL_Event event;
+    PyGILState_STATE gstate = PyGILState_Ensure();
+
+    dict = PyDict_New();
+    if (dict) {
+        if (type >= PGE_USEREVENT && type < PG_NUMEVENTS) {
+            dictcode = PyInt_FromLong(code);
+            PyDict_SetItemString(dict, "code", dictcode);
+            Py_DECREF(dictcode);
+        }
+        e = (pgEventObject *)pgEvent_New2(type, dict);
+        Py_DECREF(dict);
+
+        if (e) {
+            pgEvent_FillUserEvent(e, &event);
+#if IS_SDLv1
+            if (SDL_PushEvent(&event) < 0)
+#else
+            if (SDL_PushEvent(&event) <= 0)
+#endif
+                Py_DECREF(dict);
+            Py_DECREF(e);
+        }
+    }
+    PyGILState_Release(gstate);
+}
+
 static void
 endsound_callback(int channel)
 {
     if (channeldata) {
-        if (channeldata[channel].endevent && SDL_WasInit(SDL_INIT_VIDEO)) {
-            SDL_Event e;
-            memset(&e, 0, sizeof(e));
-            e.type = channeldata[channel].endevent;
-            if (e.type >= PGE_USEREVENT && e.type < PG_NUMEVENTS)
-                e.user.code = channel;
-            SDL_PushEvent(&e);
-        }
+        if (channeldata[channel].endevent && SDL_WasInit(SDL_INIT_VIDEO))
+            _pg_push_mixer_event(channeldata[channel].endevent, channel);
+
         if (channeldata[channel].queue) {
             PyGILState_STATE gstate = PyGILState_Ensure();
             int channelnum;
@@ -295,51 +322,6 @@ endsound_callback(int channel)
     }
 }
 
-static void
-pgMixer_AutoQuit(void)
-{
-    int i;
-    if (SDL_WasInit(SDL_INIT_AUDIO)) {
-        Py_BEGIN_ALLOW_THREADS;
-        Mix_HaltMusic();
-        Py_END_ALLOW_THREADS;
-
-        if (channeldata) {
-            for (i = 0; i < numchanneldata; ++i) {
-                Py_XDECREF(channeldata[i].sound);
-                Py_XDECREF(channeldata[i].queue);
-            }
-            free(channeldata);
-            channeldata = NULL;
-            numchanneldata = 0;
-        }
-
-        if (current_music) {
-            if (*current_music) {
-                Py_BEGIN_ALLOW_THREADS;
-                Mix_FreeMusic(*current_music);
-                Py_END_ALLOW_THREADS;
-                *current_music = NULL;
-            }
-            current_music = NULL;
-        }
-        if (queue_music) {
-            if (*queue_music) {
-                Py_BEGIN_ALLOW_THREADS;
-                Mix_FreeMusic(*queue_music);
-                Py_END_ALLOW_THREADS;
-                *queue_music = NULL;
-            }
-            queue_music = NULL;
-        }
-
-        Py_BEGIN_ALLOW_THREADS;
-        Mix_CloseAudio();
-        Py_END_ALLOW_THREADS;
-        SDL_QuitSubSystem(SDL_INIT_AUDIO);
-    }
-}
-
 static PyObject*
 import_music(void)
 {
@@ -357,6 +339,7 @@ _init(int freq, int size, int channels, int chunk, char *devicename, int allowed
     Uint16 fmt = 0;
     int i;
     PyObject *music;
+    char *drivername;
 
     if (!freq) {
         freq = request_frequency;
@@ -392,8 +375,7 @@ _init(int freq, int size, int channels, int chunk, char *devicename, int allowed
             case 6:
                 break;
             default:
-                PyErr_SetString(PyExc_ValueError, "'channels' must be 1, 2, 4, or 6");
-                return NULL;
+                return RAISE(PyExc_ValueError, "'channels' must be 1, 2, 4, or 6");
         }
     }
 #endif /* IS_SDLv2 */
@@ -436,13 +418,10 @@ _init(int freq, int size, int channels, int chunk, char *devicename, int allowed
     /*make chunk a power of 2*/
     for (i = 0; 1 << i < chunk; ++i)
         ;                     /*yes, semicolon on for loop*/
-    chunk = MAX(1 << i, 256); /*do this after foo loop exits*/
+    chunk = MAX(1 << i, 256); /*do this after for loop exits*/
 
     if (!SDL_WasInit(SDL_INIT_AUDIO)) {
-        pg_RegisterQuit(pgMixer_AutoQuit);
-
-        if (!channeldata) /*should always be null*/
-        {
+        if (!channeldata) { /*should always be null*/
             channeldata = (struct ChannelData *)malloc(
                 sizeof(struct ChannelData) * MIX_CHANNELS);
             if (!channeldata) {
@@ -456,27 +435,33 @@ _init(int freq, int size, int channels, int chunk, char *devicename, int allowed
             }
         }
 
-        if (SDL_InitSubSystem(SDL_INIT_AUDIO) == -1)
-            return PyInt_FromLong(0);
+#if IS_SDLv2
+        /* Compatibility:
+            pulse and dsound audio drivers were renamed in SDL2,
+            and we don't want it to fail.
+        */
+        drivername = SDL_getenv("SDL_AUDIODRIVER");
+        if (drivername && SDL_strncasecmp("pulse", drivername, SDL_strlen(drivername)) == 0) {
+            SDL_setenv("SDL_AUDIODRIVER", "pulseaudio", 1);
+        }
+        else if (drivername && SDL_strncasecmp("dsound", drivername, SDL_strlen(drivername)) == 0) {
+            SDL_setenv("SDL_AUDIODRIVER", "directsound", 1);
+        }
+#endif
+
+        if (SDL_InitSubSystem(SDL_INIT_AUDIO))
+            return RAISE(pgExc_SDLError, SDL_GetError());
 
 #if SDL_MIXER_MAJOR_VERSION >= 2 && SDL_MIXER_MINOR_VERSION >= 0 && SDL_MIXER_PATCHLEVEL >= 2
         if (Mix_OpenAudioDevice(freq, fmt, channels, chunk, devicename,
                                 allowedchanges) == -1) {
-            /* mixer.init swallows errors, tell user a reason via a log. */
-            if(devicename) {
-                SDL_Log("Failed to open devicename:%s: with error :%s:", devicename, SDL_GetError());
-            }
-            SDL_QuitSubSystem(SDL_INIT_AUDIO);
-            return PyInt_FromLong(0);
-        }
 #else
         if (Mix_OpenAudio(freq, fmt, channels, chunk) == -1) {
-            SDL_QuitSubSystem(SDL_INIT_AUDIO);
-            return PyInt_FromLong(0);
-        }
 #endif
+            SDL_QuitSubSystem(SDL_INIT_AUDIO);
+            return RAISE(pgExc_SDLError, SDL_GetError());;
+        }
         Mix_ChannelFinished(endsound_callback);
-
         Mix_VolumeMusic(127);
     }
 
@@ -501,25 +486,59 @@ _init(int freq, int size, int channels, int chunk, char *devicename, int allowed
         PyErr_Clear();
     }
 
-    return PyInt_FromLong(1);
+    Py_RETURN_NONE;
 }
 
 static PyObject *
-pgMixer_AutoInit(PyObject *self, PyObject *arg)
+pgMixer_AutoInit(PyObject *self)
 {
-    int freq = 0, size = 0, channels = 0, chunk = 0;
-    int allowedchanges = -1;
-
-    if (!PyArg_ParseTuple(arg, "|iiiii", &freq, &size, &channels, &chunk, &allowedchanges))
-        return NULL;
-
-    return _init(freq, size, channels, chunk, NULL, allowedchanges);
+    /* Return init with defaults */
+    return _init(0, 0, 0, 0, NULL, -1);
 }
 
 static PyObject *
 quit(PyObject *self)
 {
-    pgMixer_AutoQuit();
+    int i;
+    if (SDL_WasInit(SDL_INIT_AUDIO)) {
+        Py_BEGIN_ALLOW_THREADS;
+        Mix_HaltMusic();
+        Py_END_ALLOW_THREADS;
+
+        if (channeldata) {
+            for (i = 0; i < numchanneldata; ++i) {
+                Py_XDECREF(channeldata[i].sound);
+                Py_XDECREF(channeldata[i].queue);
+            }
+            free(channeldata);
+            channeldata = NULL;
+            numchanneldata = 0;
+        }
+
+        if (current_music) {
+            if (*current_music) {
+                Py_BEGIN_ALLOW_THREADS;
+                Mix_FreeMusic(*current_music);
+                Py_END_ALLOW_THREADS;
+                *current_music = NULL;
+            }
+            current_music = NULL;
+        }
+        if (queue_music) {
+            if (*queue_music) {
+                Py_BEGIN_ALLOW_THREADS;
+                Mix_FreeMusic(*queue_music);
+                Py_END_ALLOW_THREADS;
+                *queue_music = NULL;
+            }
+            queue_music = NULL;
+        }
+
+        Py_BEGIN_ALLOW_THREADS;
+        Mix_CloseAudio();
+        Py_END_ALLOW_THREADS;
+        SDL_QuitSubSystem(SDL_INIT_AUDIO);
+    }
     Py_RETURN_NONE;
 }
 
@@ -528,25 +547,22 @@ init(PyObject *self, PyObject *args, PyObject *keywds)
 {
     int freq = 0, size = 0, channels = 0, chunk = 0, allowedchanges = -1;
     char *devicename = NULL;
-    PyObject *result;
-    int value;
 
-    static char *kwids[] = {"frequency", "size",       "channels",
-                            "buffer",    "devicename", "allowedchanges", NULL};
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "|iiiisi", kwids, &freq,
+    static char *kwids[] = {
+        "frequency",
+        "size",
+        "channels",
+        "buffer",
+        "devicename",
+        "allowedchanges",
+        NULL
+    };
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "|iiiizi", kwids, &freq,
                                      &size, &channels, &chunk, &devicename,
                                      &allowedchanges)) {
         return NULL;
     }
-    result = _init(freq, size, channels, chunk, devicename, allowedchanges);
-    if (!result)
-        return NULL;
-    value = PyObject_IsTrue(result);
-    Py_DECREF(result);
-    if (!value)
-        return RAISE(pgExc_SDLError, SDL_GetError());
-
-    Py_RETURN_NONE;
+    return _init(freq, size, channels, chunk, devicename, allowedchanges);
 }
 
 static PyObject *
@@ -572,7 +588,6 @@ pre_init(PyObject *self, PyObject *args, PyObject *keywds)
 {
     static char *kwids[] = {"frequency", "size",       "channels",
                             "buffer",    "devicename", "allowedchanges", NULL};
-    int dname_size = 0;
 
     request_frequency = 0;
     request_size = 0;
@@ -581,8 +596,8 @@ pre_init(PyObject *self, PyObject *args, PyObject *keywds)
     request_devicename = NULL;
     request_allowedchanges = -1;
     if (!PyArg_ParseTupleAndKeywords(
-            args, keywds, "|iiiiz#i", kwids, &request_frequency, &request_size,
-            &request_channels, &request_chunksize, &request_devicename, &dname_size,
+            args, keywds, "|iiiizi", kwids, &request_frequency, &request_size,
+            &request_channels, &request_chunksize, &request_devicename,
             &request_allowedchanges))
         return NULL;
     if (!request_frequency) {
@@ -975,7 +990,9 @@ sound_dealloc(pgSoundObject *self)
 }
 
 static PyTypeObject pgSound_Type = {
-    TYPE_HEAD(NULL, 0) "Sound", sizeof(pgSoundObject), 0,
+    PyVarObject_HEAD_INIT(NULL,0)
+    "Sound",
+    sizeof(pgSoundObject), 0,
     (destructor)sound_dealloc, 0, 0, 0, /* setattr */
     0,                                  /* compare */
     0,                                  /* repr */
@@ -1085,7 +1102,7 @@ chan_get_busy(PyObject *self)
     int channelnum = pgChannel_AsInt(self);
     MIXER_INIT_CHECK();
 
-    return PyInt_FromLong(Mix_Playing(channelnum));
+    return PyBool_FromLong(Mix_Playing(channelnum));
 }
 
 static PyObject *
@@ -1290,7 +1307,8 @@ channel_dealloc(PyObject *self)
 }
 
 static PyTypeObject pgChannel_Type = {
-    TYPE_HEAD(NULL, 0) "Channel", /* name */
+    PyVarObject_HEAD_INIT(NULL,0)
+    "Channel",                    /* name */
     sizeof(pgChannelObject),      /* basic size */
     0,                            /* itemsize */
     channel_dealloc,              /* dealloc */
@@ -1372,23 +1390,24 @@ set_num_channels(PyObject *self, PyObject *args)
 static PyObject *
 set_reserved(PyObject *self, PyObject *args)
 {
-    int numchans;
-    if (!PyArg_ParseTuple(args, "i", &numchans))
+    int numchans_requested;
+    int numchans_reserved;
+    if (!PyArg_ParseTuple(args, "i", &numchans_requested))
         return NULL;
 
     MIXER_INIT_CHECK();
 
-    Mix_ReserveChannels(numchans);
-    Py_RETURN_NONE;
+    numchans_reserved = Mix_ReserveChannels(numchans_requested);
+    return PyInt_FromLong(numchans_reserved);
 }
 
 static PyObject *
 get_busy(PyObject *self)
 {
     if (!SDL_WasInit(SDL_INIT_AUDIO))
-        return PyInt_FromLong(0);
+        return PyBool_FromLong(0);
 
-    return PyInt_FromLong(Mix_Playing(-1));
+    return PyBool_FromLong(Mix_Playing(-1));
 }
 
 static PyObject *
@@ -1403,11 +1422,14 @@ Channel(PyObject *self, PyObject *args)
 }
 
 static PyObject *
-mixer_find_channel(PyObject *self, PyObject *args)
+mixer_find_channel(PyObject *self, PyObject *args, PyObject *kwargs)
 {
     int chan, force = 0;
-    if (!PyArg_ParseTuple(args, "|i", &force))
+    static char *keywords[] = {"force", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|i", keywords, &force)) {
         return NULL;
+    }
 
     MIXER_INIT_CHECK();
 
@@ -1893,7 +1915,7 @@ LOAD_BUFFER:
 }
 
 static PyMethodDef _mixer_methods[] = {
-    {"__PYGAMEinit__", pgMixer_AutoInit, METH_VARARGS,
+    {"__PYGAMEinit__", (PyCFunction)pgMixer_AutoInit, METH_NOARGS,
      "auto initialize for mixer"},
     {"init", (PyCFunction)init, METH_VARARGS | METH_KEYWORDS,
      DOC_PYGAMEMIXERINIT},
@@ -1909,7 +1931,8 @@ static PyMethodDef _mixer_methods[] = {
 
     {"get_busy", (PyCFunction)get_busy, METH_NOARGS, DOC_PYGAMEMIXERGETBUSY},
     {"Channel", Channel, METH_VARARGS, DOC_PYGAMEMIXERCHANNEL},
-    {"find_channel", mixer_find_channel, METH_VARARGS,
+    {"find_channel", (PyCFunction)mixer_find_channel,
+     METH_VARARGS | METH_KEYWORDS,
      DOC_PYGAMEMIXERFINDCHANNEL},
     {"fadeout", mixer_fadeout, METH_VARARGS, DOC_PYGAMEMIXERFADEOUT},
     {"stop", (PyCFunction)mixer_stop, METH_NOARGS, DOC_PYGAMEMIXERSTOP},
@@ -1986,6 +2009,10 @@ MODINIT_DEFINE(mixer)
     if (PyErr_Occurred()) {
         MODINIT_ERROR;
     }
+    import_pygame_event();
+    if (PyErr_Occurred()) {
+        MODINIT_ERROR;
+    }
 
     /* type preparation */
     if (PyType_Ready(&pgSound_Type) < 0) {
@@ -2029,8 +2056,6 @@ MODINIT_DEFINE(mixer)
     c_api[2] = pgSound_Play;
     c_api[3] = &pgChannel_Type;
     c_api[4] = pgChannel_New;
-    c_api[5] = pgMixer_AutoInit;
-    c_api[6] = pgMixer_AutoQuit;
     apiobj = encapsulate_api(c_api, "mixer");
     if (apiobj == NULL) {
         DECREF_MOD(module);
