@@ -46,6 +46,8 @@ typedef struct {
 static const char pg_default_encoding[] = "unicode_escape";
 static const char pg_default_errors[] = "backslashreplace";
 
+static PyObject* os_module = NULL;
+
 #define PATHLIB "pathlib"
 #define PUREPATH "PurePath"
 
@@ -314,6 +316,17 @@ pgRWops_IsFileObject(SDL_RWops *rw)
     return rw->close == _pg_rw_close;
 }
 
+char*
+pgRWops_GetFileExtension(SDL_RWops* rw)
+{
+    if (pgRWops_IsFileObject(rw)) {
+        return NULL;
+    }
+    else {
+        return rw->hidden.unknown.data1;
+    }
+}
+
 #if IS_SDLv2
 static Sint64
 _pg_rw_size(SDL_RWops *context)
@@ -357,7 +370,8 @@ _pg_rw_size(SDL_RWops *context)
         PyErr_Print();
         goto end;
     }
-    size = PyInt_AsLong(tmp);
+
+    size = PyLong_AsLongLong(tmp);
     if (size == -1 && PyErr_Occurred() != NULL) {
         PyErr_Print();
         goto end;
@@ -404,9 +418,11 @@ _pg_rw_write(SDL_RWops *context, const void *ptr, size_t size, size_t num)
         return -1;
 
 #if PY3
-    result = PyObject_CallFunction(helper->write, "y#", ptr, size * num);
+    result = PyObject_CallFunction(helper->write, "y#", (const char *)ptr,
+                                    (Py_ssize_t)size * num);
 #else  /* PY2 */
-    result = PyObject_CallFunction(helper->write, "s#", ptr, size * num);
+    result = PyObject_CallFunction(helper->write, "s#", (const char *)ptr,
+                                    (Py_ssize_t)size * num);
 #endif  /* PY2 */
     if (!result)
         return -1;
@@ -424,9 +440,11 @@ _pg_rw_write(SDL_RWops *context, const void *ptr, size_t size, size_t num)
     state = PyGILState_Ensure();
 
 #if PY3
-    result = PyObject_CallFunction(helper->write, "y#", ptr, size * num);
+    result = PyObject_CallFunction(helper->write, "y#", (const char *)ptr,
+                                    (Py_ssize_t)size * num);
 #else  /* PY2 */
-    result = PyObject_CallFunction(helper->write, "s#", ptr, size * num);
+    result = PyObject_CallFunction(helper->write, "s#", (const char *)ptr,
+                                    (Py_ssize_t)size * num);
 #endif  /* PY2 */
     if (!result) {
         PyErr_Print();
@@ -509,6 +527,9 @@ pgRWops_FromFileObject(PyObject *obj)
     helper->file = obj;
     Py_INCREF(obj);
 
+    /* Adding a helper to the hidden data to support file-like object RWops
+     * RWops from actual files use this space to store the file extension
+     * for later use */
     rw->hidden.unknown.data1 = (void *)helper;
 #if IS_SDLv2
     rw->size = _pg_rw_size;
@@ -567,6 +588,7 @@ pgRWops_ReleaseObject(SDL_RWops *context)
 #endif /* WITH_THREAD */
     }
     else {
+        free(context->hidden.unknown.data1);
         ret = SDL_RWclose(context);
         if (ret < 0)
             PyErr_SetString(PyExc_IOError, SDL_GetError());
@@ -604,7 +626,8 @@ _pg_rw_seek(SDL_RWops *context, Sint64 offset, int whence)
     if (!(offset == 0 &&
           whence == SEEK_CUR)) /* being seek'd, not just tell'd */
     {
-        result = PyObject_CallFunction(helper->seek, "ii", offset, whence);
+        result = PyObject_CallFunction(helper->seek, "Li",
+                                    (long long)offset, whence);
         if (!result) {
             PyErr_Print();
             retval = -1;
@@ -620,7 +643,10 @@ _pg_rw_seek(SDL_RWops *context, Sint64 offset, int whence)
         goto end;
     }
 
-    retval = PyInt_AsLong(result);
+    retval = PyLong_AsLongLong(result);
+    if (retval == -1 && PyErr_Occurred())
+        PyErr_Clear();
+
     Py_DECREF(result);
 
 end:
@@ -637,7 +663,8 @@ end:
 
     if (!(offset == 0 && whence == SEEK_CUR)) /*being called only for 'tell'*/
     {
-        result = PyObject_CallFunction(helper->seek, "ii", offset, whence);
+        result = PyObject_CallFunction(helper->seek, "Li",
+                                    (long long)offset, whence);
         if (!result)
             return -1;
         Py_DECREF(result);
@@ -647,7 +674,10 @@ end:
     if (!result)
         return -1;
 
-    retval = PyInt_AsLong(result);
+    retval = PyLong_AsLongLong(result);
+    if (retval == -1 && PyErr_Occurred())
+        PyErr_Clear();
+
     Py_DECREF(result);
 
     return retval;
@@ -688,7 +718,8 @@ _pg_rw_read(SDL_RWops *context, void *ptr, size_t size, size_t maxnum)
 #ifdef WITH_THREAD
     state = PyGILState_Ensure();
 #endif /* WITH_THREAD */
-    result = PyObject_CallFunction(helper->read, "i", size * maxnum);
+    result = PyObject_CallFunction(helper->read, "K",
+                                    (unsigned long long)size * maxnum);
     if (!result) {
         PyErr_Print();
         retval = -1;
@@ -703,8 +734,10 @@ _pg_rw_read(SDL_RWops *context, void *ptr, size_t size, size_t maxnum)
     }
 
     retval = Bytes_GET_SIZE(result);
-    memcpy(ptr, Bytes_AsString(result), retval);
-    retval /= size;
+    if (retval) {
+        memcpy(ptr, Bytes_AsString(result), retval);
+        retval /= size;
+    }
 
     Py_DECREF(result);
 
@@ -722,22 +755,83 @@ _rwops_from_pystr(PyObject *obj)
     if (obj != NULL) {
         SDL_RWops *rw = NULL;
         PyObject *oencoded;
+        char* encoded = NULL;
+        char* ext = NULL;
+        char* extension = NULL;
+
         oencoded = pg_EncodeString(obj, "UTF-8", NULL, NULL);
         if (oencoded == NULL) {
             return NULL;
         }
         if (oencoded != Py_None) {
-            rw = SDL_RWFromFile(Bytes_AS_STRING(oencoded), "rb");
+            encoded = Bytes_AS_STRING(oencoded);
+            rw = SDL_RWFromFile(encoded, "rb");
+            ext = strrchr(encoded, '.');        
+            if (ext && strlen(ext) > 1) {
+                ext++;
+                extension = malloc(strlen(ext)+1);
+                if (extension == NULL) {
+                    return (SDL_RWops*)PyErr_NoMemory();
+                }
+                strcpy(extension, ext);
+            }
         }
         Py_DECREF(oencoded);
         if (rw) {
+            /* adding the extension to the hidden data for RWops from files */
+            /* this is necessary to support loading functions that rely on
+             * file extensions in a convenient way. File-like objects use this
+             * field for a helper object. */
+            rw->hidden.unknown.data1 = (void *)extension;
             return rw;
         } else {
 #if PY3
             if (PyUnicode_Check(obj)) {
                 SDL_ClearError();
-                PyErr_SetString(PyExc_FileNotFoundError,
-                                "No such file or directory.");
+
+                if (os_module) {
+                    PyObject* cwd = PyObject_CallMethod(os_module, "getcwd",
+                                                        NULL);
+                    if (cwd == NULL) {
+                        PyErr_SetString(PyExc_FileNotFoundError, 
+                                        "No such file or directory.");
+                        return NULL;
+                    }
+
+                    PyObject* path = PyObject_GetAttrString(os_module, "path");
+                    if (path == NULL) {
+                        Py_DECREF(cwd);
+                        PyErr_SetString(PyExc_FileNotFoundError,
+                                        "No such file or directory.");
+                        return NULL;                        
+                    }
+
+                    PyObject* isabs = PyObject_CallMethod(path, "isabs", "O", obj);
+                    if (isabs == NULL) {
+                        Py_DECREF(cwd);
+                        Py_DECREF(path);
+                        PyErr_SetString(PyExc_FileNotFoundError,
+                                        "No such file or directory.");
+                        return NULL;
+                    }
+
+                    if (isabs == Py_False) {
+                        PyErr_Format(PyExc_FileNotFoundError,
+                                     "No file '%S' found in working directory"
+                                     " '%S'.", obj, cwd);                       
+                    }
+                    else {
+                        PyErr_Format(PyExc_FileNotFoundError, 
+                                     "No such file or directory: '%S'.", obj);
+                    }
+                    Py_DECREF(cwd);
+                    Py_DECREF(path);
+                    Py_DECREF(isabs);
+                }
+                else {
+                    PyErr_Format(PyExc_FileNotFoundError, 
+                                 "No such file or directory: '%S'.", obj);
+                }
 #else
             if (PyUnicode_Check(obj) || PyString_Check(obj)) {
                 SDL_ClearError();
@@ -850,6 +944,7 @@ MODINIT_DEFINE(rwobject)
     c_api[3] = pg_EncodeString;
     c_api[4] = pgRWops_FromFileObject;
     c_api[5] = pgRWops_ReleaseObject;
+    c_api[6] = pgRWops_GetFileExtension;
     apiobj = encapsulate_api(c_api, "rwobject");
     if (apiobj == NULL) {
         DECREF_MOD(module);
@@ -861,5 +956,11 @@ MODINIT_DEFINE(rwobject)
         DECREF_MOD(module);
         MODINIT_ERROR;
     }
+
+    /* import os, don't sweat if it errors, it will be checked before use */
+    os_module = PyImport_ImportModule("os");
+    if (os_module == NULL)
+        PyErr_Clear();
+
     MODINIT_RETURN(module);
 }
